#pragma once

#include <amdis/common/TypeTraits.hpp>
#include <amdis/linearalgebra/PreconditionerInterface.hpp>

namespace AMDiS
{
  class SolverInfo;

  /// Interface for Runner / Worker types used in solver classes
  template <class Traits>
  class RunnerInterface
  {
  protected:
    using PreconBase = PreconditionerInterface<Traits>;
    using Mat = typename Traits::Mat;
    using Sol = typename Traits::Sol;
    using Rhs = typename Traits::Rhs;
    using Comm = typename Traits::Comm;

  public:
    /// virtual destructor
    virtual ~RunnerInterface() = default;

    /// Is called at the beginning of a solution procedure
    virtual void init(Mat const& A, Comm& comm) = 0;

    /// Is called at the end of a solution procedure
    virtual void exit() = 0;

    /// Solve the system A*x = b
    virtual int solve(Mat const& A, Sol& x, Rhs const& b, SolverInfo& solverInfo) = 0;

    /// Solve the adjoint system A^T*x = b
    virtual int adjointSolve(Mat const& A, Sol& x, Rhs const& b, SolverInfo& solverInfo)
    {
      error_exit("Must be implemented by derived class.");
      return 0;
    }

    virtual std::shared_ptr<PreconBase> leftPrecon() { return {}; }
    virtual std::shared_ptr<PreconBase> rightPrecon() { return {}; }
  };

} // end namespace AMDiS
