#pragma once

#include <list>

#include <Eigen/SparseCore>

#include <amdis/common/Index.hpp>
#include <amdis/linearalgebra/Common.hpp>
#include <amdis/linearalgebra/Constraints.hpp>

namespace AMDiS
{
  template <class T, int O>
  struct Constraints<Eigen::SparseMatrix<T,O>>
  {
    using Matrix = Eigen::SparseMatrix<T,O>;

    template <class BitVector>
    static std::vector<Triplet<T>> dirichletBC(Matrix& mat, BitVector const& nodes, bool setDiagonal = true)
    {
      clearDirichletRow(mat, nodes, setDiagonal, int_<O>);
      return {};
    }

    template <class BitVector, class Associations>
    static std::vector<Triplet<T>> periodicBC(Matrix& mat, BitVector const& left, Associations const& left2right,
      bool setDiagonal = true)
    {
      error_exit("Not implemented");
      return {};
    }

  protected:
    template <class BitVector>
    static void clearDirichletRow(Matrix& mat, BitVector const& nodes, bool setDiagonal, int_t<Eigen::ColMajor>)
    {
      for (typename Matrix::Index c = 0; c < mat.outerSize(); ++c) {
        for (typename Matrix::InnerIterator it(mat, c); it; ++it) {
          if (nodes[it.row()]) {
            it.valueRef() = (setDiagonal && it.row() == it.col() ? T(1) : T(0));
            break;
          }
        }
      }
    }

    template <class BitVector>
    static void clearDirichletRow(Matrix& mat, BitVector const& nodes, bool setDiagonal, int_t<Eigen::RowMajor>)
    {
      for (typename Matrix::Index r = 0; r < mat.outerSize(); ++r) {
        if (nodes[r]) {
          for (typename Matrix::InnerIterator it(mat, r); it; ++it) {
            it.valueRef() = (setDiagonal && it.row() == it.col() ? T(1) : T(0));
          }
        }
      }
    }
  };

} // end namespace AMDiS
